#--------------------------------------------------------------------
#                     TRAFFIC LIGHTS CONTROLLER
#                     =========================
#
# This is a traffic lights controller project controlling lights
# at a junction. 6 LEDS are used to represent the traffic lights.
# Additionally a button is used for pedestrian crossing, and an
# LCD shows the state of the traffic lights at any time
#
# Author: Dogan Ibrahim
# File  : traffic.py
# Date  : May 2020
#----------------------------------------------------------------------
import RPi.GPIO as GPIO				# Import RPi
import multiprocessing				# Import multiprocessing
import time					# Import time
import RPi_I2C_driver				# I2C library
LCD = RPi_I2C_driver.lcd()			# Import LCD
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)				# GPIO mode BCM
pedq = multiprocessing.Queue()			# Create queue
lcdq = multiprocessing.Queue()			# Creaet queue

#
# This function sends data 'state (0 or 1)' to specified port
#
def ONOF(port, state):
   GPIO.output(port, state)

#
# This function configures the specified port as output
#
def CONF_OUT(port):
   GPIO.setup(port, GPIO.OUT)

#
# Process to control the lights
#
def Lights():					# Process Lights
   R1=21; A1=20; G1=16				# LED connections
   R2=12; A2=7;  G2=8				# LED conenctions
   Buzzer=25					# Buzzer connection
   CONF_OUT(R1); CONF_OUT(A1); CONF_OUT(G1)	# Configure
   CONF_OUT(R2); CONF_OUT(A2); CONF_OUT(G2)	# Configure
   CONF_OUT(Buzzer)				# Configure
   ONOF(R1,0); ONOF(A1,0); ONOF(G1,0); ONOF(R2,0); ONOF(A2,0); ONOF(G2,0)
   ONOF(Buzzer, 0)

   RedDuration = 15
   GreenDuration = 15
   AmberDuration = 2

#
# Control the traffic light sequence
#
   while True:					# Do forever
      ONOF(R1,0); ONOF(A1,0); ONOF(G1,1); ONOF(R2,1); ONOF(A2,0); ONOF(G2,0)
      time.sleep(RedDuration)
      ONOF(G1,0); ONOF(A1,1)
      time.sleep(AmberDuration)
      ONOF(A1,0); ONOF(R1,1); ONOF(A2,1)
      time.sleep(AmberDuration)
      ONOF(A2,0); ONOF(R2,0); ONOF(G2,1)
      time.sleep(GreenDuration)
      ONOF(G2,0); ONOF(A2,1)
      time.sleep(AmberDuration)
      ONOF(A2,0); ONOF(A1,1); ONOF(R2,1)
      time.sleep(AmberDuration)

      while not pedq.empty():			# If ped request
         lcdq.put(1)
         ONOF(G1,0); ONOF(R1,1); ONOF(A1,0)	# Only RED ON
         ONOF(G2,0); ONOF(R2,1); ONOF(A2,0)	# Only RED ON
         d = pedq.get()				# Clear ledq
         ONOF(Buzzer, 1)			# Buzzer ON
         time.sleep(10)				# Wait 10 secs
         ONOF(Buzzer, 0)			# Buzzer OFF
         d = lcdq.get()				# Clear lcdq
      

def Pedestrian():				# Process Pedestrian
   PB1 = 24
   GPIO.setup(PB1, GPIO.IN)			# PB1 is input

   while True:					# Do forever
      while GPIO.input(PB1) == 1:		# PB1 not pressed
         pass
      pedq.put(1)				# Send to Ped queue
      while GPIO.input(PB1) == 0:		# PB1 not released
         pass

#
# Create the processes
#
p = multiprocessing.Process(target = Lights, args = ())
q = multiprocessing.Process(target = Pedestrian, args = ())
p.start()
q.start()

#
# LCD Display control. Display 'Ped Cycle' or 'Traffic Cycle'
#
LCD.lcd_clear()					# Clear LCD
LCD.lcd_display_string("TRAFFIC CONTROL", 1)	# Heading

while True:					# DO forever
   if not lcdq.empty():
      LCD.lcd_display_string("Ped Cycle    ", 2)
   else:
      LCD.lcd_display_string("Traffic Cycle", 2)
   time.sleep(1)




